import jas.engine.AgentList;
import jas.engine.Sim;
import jas.space.DblGrid;
import jas.space.ObjGrid;


/**
 *
 * <p>Title: JAS</p>
 * <p>Description: Java Agent-based Simulation library</p>
 * <p>Copyright (C) 2002 Michele Sonnessa</p>
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * @author Michele Sonnessa
 * 
 */
public class DaisyWorld {
	
	public static double solarFluxConstant = 952;
	public static double SBConstant = 5.669E-008;
	public double solarMultiplier = 1.0;
	public static double planetAlbedo = 0.5;
	
	public double mediumTemp = 35.0;
	public double periodTemp = 25.0;
	
	public DblGrid temperatureLayer;
	public ObjGrid daisyLayer;
	public int xSize, ySize;
	public AgentList daisies;
	public double sunPeriod;
	public int phase;
	
	public DaisyWorld(int xSize, int ySize, int sunPeriod, int phase) {
		this.xSize = xSize;
		this.ySize = ySize;
		temperatureLayer = new DblGrid(xSize, ySize);
		daisyLayer = new ObjGrid(xSize, ySize);
		daisies = new AgentList();
		this.sunPeriod = sunPeriod;
		this.phase = phase;
	}
	
	public void placeMeRandom(Daisy daisy)
	{
		int x = 0, y = 0;
		int cnt = 0;
		
		Daisy who = null;
		do 
		{
			cnt++;
			x = Sim.getRnd().getIntFromTo(0, xSize - 1);
			y = Sim.getRnd().getIntFromTo(0, ySize - 1);
			who = (Daisy) daisyLayer.get(x, y);			
		} while (who != null && cnt < 20);
		
		if (cnt >= 20)
			return;
			
		daisy.setXY(x, y);
		daisyLayer.set(x, y, daisy);
	}

	public void placeMeAt(int x, int y, Daisy daisy)
	{
		if (daisyLayer.get(x, y) != null)
			throw new UnsupportedOperationException("placeMeAt: The position (" + x + "," + y + ") is not empty." );
		
		daisy.setXY(x, y);
		daisyLayer.set(x, y, daisy);
	}
	
	public void insulate()
	{
		double sun = getLocalTemperature(planetAlbedo);
		
		for (int x = 0; x < xSize; x++)
			for (int y = 0; y < ySize; y++)
			{
				Daisy daisy = (Daisy) daisyLayer.get(x, y);
				if (daisy == null)
					temperatureLayer.setDbl(x, y, sun );
				else
					temperatureLayer.setDbl(x, y, getLocalTemperature( daisy.getAlbedo()) );					
			}	
	}
	
//	public double getPlanetAlbedo()
//	{
//		double albedo = 0.0;
//		
//		for (int x = 0; x < xSize; x++)
//			for (int y = 0; y < ySize; y++)
//			{
//				Daisy daisy = (Daisy) daisyLayer.get(x, y);
//				if (daisy == null)
//					albedo += 0.5;
//				else
//					albedo += daisy.getAlbedo();
//			}	
//			
//			return albedo / (xSize * ySize);
//	}
		
	public double getLocalTemperature(double albedo)
	{
		//return Math.sin(phase * Sim.getAbsoluteTime() * Math.PI / sunPeriod) * periodTemp + mediumTemp;
		
		double solarLuminosity = (solarMultiplier + (Math.sin(Sim.getAbsoluteTime() * Math.PI / sunPeriod) / 2));
		double res = Math.pow(solarLuminosity * solarFluxConstant * ( 1.0 - albedo ) / SBConstant, 0.25) - 273.0;
		//System.out.println("SUN T " + res);
		return res;
		
		//double solarLuminosity = 0.6 + (Sim.getAbsoluteTime()* (1.2/100.0));
		//return (Math.pow( (solarLuminosity * solarFluxConstant * 0.5 / SBConstant), 0.25) - 273);
	}
	
	public double getSunTemperature()
	{
		return getLocalTemperature(planetAlbedo);
	}
	
	public double getAverageTemp()
	{
		//System.out.println("SUM " + world.temperatureLayer.sum());
		return temperatureLayer.sum() / (xSize * ySize);
	}	
}
