
import jas.engine.*;
import jas.events.*;
import jas.graphics.plot.TimeSeriesPlotter;
import jas.plot.*;


/**
  *	A JAS implementation of the DaisyModel. See article for more details about the model.
  *
  *	(c) 2003 - The Turin Cluster Group, LIASES, University of Torino
  * @author Riccardo Boero (boero@econ.unito.it), Michele Sonnessa (sonnessa@di.unito.it)
  * @author Gigi Ferraris (glf313@libero.it), Matteo Morini (morini@labor-torino.it)
  */
public class GeneticDaisyObserver extends SimModel
{
	GeneticDaisyModel model;
	private LayeredSurfaceFrame display;
	private TimeSeriesPlotter plotter, daisyFight;
	
	public int white, black;
	
	public void setParameters()
	{
		model = (GeneticDaisyModel) Sim.engine.getModelWithID("GeneticDaisyModel");
	}

	private void buildActions()
	{
		SimGroupEvent s;

		s = eventList.scheduleGroup(0, 1);
		s.addEvent(display, Sim.EVENT_UPDATE);
		s.addEvent(plotter, Sim.EVENT_UPDATE);
		s.addEvent(this, "countDaisies");
		s.addEvent(daisyFight, Sim.EVENT_UPDATE);
	}

	public void buildModel()
	{
		ColorMap cRange = new ColorRangeMap(64, 0, 200, java.awt.Color.red);

		LayerDblGridDrawer d = new LayerDblGridDrawer("Heat", model.world.temperatureLayer, cRange);
		LayerObjGridDrawer s = new LayerObjGridDrawer("Bugs", model.world.daisyLayer);

		display = new LayeredSurfaceFrame(model.world.xSize, model.world.ySize, 4);
		display.addLayer(d);
		display.addLayer(s);
		addSimWindow(display);
		
		plotter = new TimeSeriesPlotter("Temperature");
		plotter.addSeries("world", model, "getAverageTemp", true);
		plotter.addSeries("sun", model.world, "getSunTemperature", true);
		addSimWindow(plotter);
		
		daisyFight = new TimeSeriesPlotter("Daisies");
		daisyFight.addSeries("shining", this, "white", false);
		daisyFight.addSeries("dark", this, "black", false);
		addSimWindow(daisyFight);
		
		buildActions();
	}

	public void countDaisies()
	{
		black = 0;
		white = 0;
		
		AgentList daisies = model.world.daisies;
		for (int i = 0; i < daisies.size(); i++) {
			if ( ((Daisy) daisies.get(i)).getAlbedo() > 0.5 )
				white++;
			else
				black++;		
		}
	}
}

